key = {
    "delay_list": "/xpcs/multitau/delay_list",
    "g2": "/xpcs/multitau/normalized_g2",
    "g2_err": "/xpcs/multitau/normalized_g2_err",
    "g2_partial": "/xpcs/multitau/normalized_g2_partial",
    "g2_partial_err": "/xpcs/multitau/normalized_g2_partial_err",
    "g2_partial_labels": "/xpcs/multitau/normalized_g2_partial_labels",
    "G2IPIF": "/xpcs/multitau/unnormalized_G2",
    "multitau_config": "/xpcs/multitau/config",
    "saxs_2d": "/xpcs/temporal_mean/scattering_2d",
    "saxs_sq": "/xpcs/temporal_mean/scattering_sq",
    "saxs_1d": "/xpcs/temporal_mean/scattering_1d",
    "saxs_1d_segments": "/xpcs/temporal_mean/scattering_1d_segments",
    "intensity_vs_time": "/xpcs/spatial_mean/intensity_vs_time",
    "c2_prefix": "/xpcs/twotime",
    "c2_g2": "/xpcs/twotime/normalized_g2",
    "c2_g2_segments": "/xpcs/twotime/normalized_g2_segments",
    "c2_delay": "/xpcs/twotime/delay_list",
    "twotime_config": "/xpcs/twotime/config",
    "processed_bins": "/xpcs/twotime/processed_bins",
    "rawdata_path": "/xpcs/rawdata_path",
}
